 /*
   Authors: Loïc Besnard, Patricia Bournai, Thierry Gautier, Paul Le Guernic.
   Copyright (C) by INRIA-CNRS - developed by Espresso project - GPL v2 License - 2010.

   Contact: polychronycontact@listes.irisa.fr

   This file is part of Polychrony Signal GUI (http://www.irisa.fr/espresso/polychrony).

   Polychrony Signal GUI is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License Version 2 as published by the Free
   Software Foundation.

   Polychrony Signal GUI is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License Version 2
   for more details.

   You should have received a copy of the GNU General Public License Version 2
   along with Polychrony Signal GUI; see the file COPYING. If not, consult the url 
    http://www.gnu.org/licenses/gpl-2.0.html, or write to the Free
   Software Foundation, 51 Franklin Street, Fifth Floor, Boston, MA
   02110-1301, USA.
 */


package common;


import java.awt.Container;
import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.Color;
import java.awt.Font;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.JButton;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTextArea;
import javax.swing.BorderFactory;  // Not for Eclipse.

/// A window to display the diagnostic of the Signal compiler
public class CompilerDiagnostic extends JFrame {
    private static final long serialVersionUID = 1L;
    private CompilerWin compWin; // reference of the compiling window
    private JTextArea text;

    // Constructor
    public CompilerDiagnostic(CompilerWin cw, int x0, int y0) {
        this.compWin = cw;
        Container frame = getContentPane();
        frame.setLayout(new BorderLayout());
        // build of the label
        JLabel title = new JLabel("         DIAGNOSTICS ");
        setTitle("Compiler Diagnostics");
        title.setForeground(Color.black);
        title.setBorder(BorderFactory.createMatteBorder(20, 20, 20, 20, pKGUIColors.Color1));
        title.setFont(new Font("Helvetica", Font.PLAIN, 20));

        // build of the text area
        text = new JTextArea();
        JScrollPane scrollArea = new JScrollPane(text);
        scrollArea
        .setVerticalScrollBarPolicy(JScrollPane.VERTICAL_SCROLLBAR_ALWAYS);
        scrollArea.setPreferredSize(new Dimension(400, 400));

        // build of the END button
        JPanel endPanel = new JPanel();
        JButton endButton = new JButton("END");
        MyDiagListener myListener = new MyDiagListener(this);
        endButton.addActionListener(myListener);
        endPanel.add(endButton, BorderLayout.CENTER);

        frame.add(title, BorderLayout.NORTH);
        frame.add(scrollArea, BorderLayout.CENTER);
        frame.add(endPanel, BorderLayout.SOUTH);
        setLocation(x0, y0);
        pack();
    }

    // Insert the string ?s in the diagnostic window
    public void append(String s) {
        text.append(s);
    }

    // Reset the diagnostic window
    public void reset() {
        text.setText("");
    }

    public CompilerWin getCompilerWin() {
        return compWin;
    }
}

/** Listener associated with the END button */
class MyDiagListener implements ActionListener {
    CompilerDiagnostic parent;

    public MyDiagListener(CompilerDiagnostic w) {
        parent = w;
    }

    public void actionPerformed(ActionEvent e) {
        parent.setVisible(false);
    }
}
