 /*
   Authors: Loïc Besnard, Patricia Bournai, Thierry Gautier, Paul Le Guernic.
   Copyright (C) by INRIA-CNRS - developed by Espresso project - GPL v2 License - 2010.

   Contact: polychronycontact@listes.irisa.fr

   This file is part of Polychrony Signal GUI (http://www.irisa.fr/espresso/polychrony).

   Polychrony Signal GUI is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License Version 2 as published by the Free
   Software Foundation.

   Polychrony Signal GUI is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License Version 2
   for more details.

   You should have received a copy of the GNU General Public License Version 2
   along with Polychrony Signal GUI; see the file COPYING. If not, consult the url 
    http://www.gnu.org/licenses/gpl-2.0.html, or write to the Free
   Software Foundation, 51 Franklin Street, Fifth Floor, Boston, MA
   02110-1301, USA.
 */


package common;


import javax.swing.JCheckBox;
import common.PolychronyWindow;
import common.CompilerFunct;

/**
 * This class manages the different windows used during a Signal program
 * compiling.
 */
public class CompilerWin {
    private static final int NOTHING = -2;
    private static final int FINISH = -1;

    private LiaisonSignal LS;
    private PolychronyWindow mainWin;
    private CompilerFunct compilerInterFunct; // window for the functionalities in interactive mode
    private CompilerFunct codeGeneratorsInteractiveMode; // window for the different code generators
    private CompilerOptions vCompilerOptions; // window for the functionalities in automatic mode
    private CompilerDiagnostic compilerDiag; // reference to the diagnostic window
    private int noFunc = NOTHING;

    private void showDebug(String message) {
//	JOptionPane.showMessageDialog(this, message, "DEBUG", JOptionPane.INFORMATION_MESSAGE);
	System.out.println("DEBUG "+message);
    }

    private void initialize(LiaisonSignal l, PolychronyWindow gw, int x0, int y0) {
	LS = l;
        mainWin = gw;
        JCheckBox[] tfunc = new JCheckBox[CompilerConst.AllFuncInteractive];
        compilerInterFunct = new CompilerFunct(l, gw, this, x0, y0, 0,
                                               CompilerConst.nbCompFunc - 1, "END", "   FUNCTIONALITIES:   ",
                                               "", "Compiler functionalities", tfunc);
        codeGeneratorsInteractiveMode = new CompilerFunct(l, gw, this, 10 * x0,
                10 * y0, CompilerConst.nbCompFunc,
                CompilerConst.AllFuncInteractive - 1, "CLOSE",
                "   OUTPUTS    ", "", "Code generators", tfunc);
        compilerDiag = new CompilerDiagnostic(this, 400, 20);
    }

    /** Constructor: it creates the windows necessary during the compiling:
	one window for the diagnostics the three functionalities windows (which 
	corresponds to the different mode of compiling.
	Warning: the "option window" is shared by all the the created windows.
    */
    public CompilerWin(LiaisonSignal l, PolychronyWindow gw, int x0, int y0) {
        initialize(l, gw, x0, y0);
        vCompilerOptions = new CompilerOptions(gw, this, x0, y0);
    }

    public CompilerWin(LiaisonSignal l, PolychronyWindow gw, int x0, int y0,
                       CompilerWin compilerWinMain) {
        initialize(l, gw, x0, y0);
        vCompilerOptions = compilerWinMain.vCompilerOptions;
    }

    // Display and reset the compiler diagnostic window
    public void displayCompilerDiag() {
        // compilerDiag.show();
        // showDebug("CompilerWin::displayCompilerDiag");
        compilerDiag.setVisible(true);
        compilerDiag.reset();
    }

    // Display the compiler (in interactive mode) functionalities window and the diagnostic window
    public void displayCompilerFunct() {
        // showDebug("CompilerWin::displayCompilerFunct");
	// compilerInterFunct.setCurrentWindow(LS);
        compilerInterFunct.setVisible(true);
        displayCompilerDiag();
    }

    // Display the compiler automatic functionalities window and the the diagnostic window
    public void displayAutoCompilerFunct() {
        // showDebug("CompilerWin::displayAutoCompiler");
        vCompilerOptions.setCurrentWindow(LS);
        vCompilerOptions.setVisible(true);
        displayCompilerDiag();
    }

    // Insert the string ?s in the diagnostic window
    public void append(String s) {
        compilerDiag.append(s);
    }

    // Get the last functionality clicked by the user
    public int getCompilerFunctionality() {
        int val = noFunc;
        noFunc = NOTHING;
        return (val);
    }

    public void compilerDiagHide() {
        compilerDiag.setVisible(false);
    }

    /**
       End of interactive compiling. This method is called when the user clicks on
       the buttons END of the diagnostic window or the functionalities window.
    */
    public void endInteractCompiling() {
        noFunc = FINISH;
        compilerDiagHide();
        compilerInterFunct.setVisible(false);
        codeGeneratorsInteractiveMode.setVisible(false);
        LS.sig_iterate();
        mainWin.reset_selectedAction();
        noFunc = NOTHING;
    }

    public CompilerFunct getCompilerFunct() {
        return compilerInterFunct;
    }

    // Applying the ith functionality
    public void applyFunctionality(int i) {
        noFunc = i;
	// showDebug("DEBUG Lbesnard: applyFunctionality: " + i);
        LS.sig_iterate();
    }

    // Button for generators
    public void itemStateChangedCodeGenerators(boolean bhide) {
        if (bhide)
            codeGeneratorsInteractiveMode.setVisible(false);
        else
            codeGeneratorsInteractiveMode.see(true);
    }
}
