 /*
   Authors: Loïc Besnard, Patricia Bournai, Thierry Gautier, Paul Le Guernic.
   Copyright (C) by INRIA-CNRS - developed by Espresso project - GPL v2 License - 2010.

   Contact: polychronycontact@listes.irisa.fr

   This file is part of Polychrony Signal GUI (http://www.irisa.fr/espresso/polychrony).

   Polychrony Signal GUI is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License Version 2 as published by the Free
   Software Foundation.

   Polychrony Signal GUI is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License Version 2
   for more details.

   You should have received a copy of the GNU General Public License Version 2
   along with Polychrony Signal GUI; see the file COPYING. If not, consult the url 
    http://www.gnu.org/licenses/gpl-2.0.html, or write to the Free
   Software Foundation, 51 Franklin Street, Fifth Floor, Boston, MA
   02110-1301, USA.
 */


package common;


import java.awt.AlphaComposite;
import java.awt.Composite;
import java.awt.Font;
import java.awt.FontMetrics;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.BasicStroke;
import java.awt.Color;
import java.awt.Image;
import java.awt.event.MouseEvent;
import java.lang.Math;
import java.util.Vector;

import javax.swing.ImageIcon;
import javax.swing.JPanel;
import javax.swing.SwingUtilities;
import javax.swing.event.MouseInputAdapter;

public class GraphCanvas extends JPanel {
    private static final long serialVersionUID = 1L;
    private static final int White = 0;
    private static final int Black = 1;
    private static final int Blue = 2;
    private static final int Red = 3;
    private static final int Green = 4;
    private static final int Purple = 5;
    private static final int Yellow = 6;
    private static final int Gray = 7;
    private static final int C_left = 0;
    private static final int C_middle = 1;
    private static final int C_right = 2;
    private static final int C_ctrl_middle = 3;
    private static final int clearwin = 0;
    private static final int cleararea = 1;
    private static final int rectangle = 2;
    private static final int polyline = 3;
    private static final int line = 4;
    private static final int fillpoly = 5;
    private static final int string = 6;

    private Font portFont; // font for the Signal declarations associated with the ports
    private Font boxFont;  // font for the thext in the boxes
    private int portH;     // Height of the ports font
    private int portL;     // Width of the ports font
    private int boxH;      // Height of the boxes font
    private int boxL;      // Width of the boxes font
    private int typeFont;  // set to J_box_police (J_port_police) if the user has clicked on the menu box font (port font)

    private PolychronyWindow this_Pwindow;
    private LiaisonSignal this_C_interface; // Link with the C world
    private boolean gridState = false;
    private int largGrid = 30;
    private int gridStroke = 1;
    private int StrokeWidth = 1; // modified by "Stroke Witdh" menu.
    private Graphics gc; //  Current graphical context comes from PaintComponent or allocated as soon as the user clicks 
    private Graphics2D gc2; //Graphical context to display with a line width > 1
    private int MouseButtonVal; // Type of the clicked button : left, right, middle...
    private boolean MouseButtonState; // True if the button has been clicked on
    private boolean isMouseMovetraited = false; // True if the mouse movements are taken into account
    private int compteur = 0;
    private double scale = 1.0;
    private int x, sw_sav, sx_sav, w_sav, h_sav, w, h;
    private int y, sh_sav, sy_sav;

    private Vector listButLib = new Vector(); //  List containing all the buttons of the libs and their images

    private Color getMyColor1() {
        return  pKGUIColors.Color1; //  new Color(200, 200, 230);
    }

    public void addLibBut(LibButton but) {
        listButLib.add(but);
    }

    public void removeLibBut(LibButton but) {
        listButLib.remove(but);
    }

    public GraphCanvas(PolychronyWindow Pw, LiaisonSignal Pw_C_interface) {
        FontMetrics fmetrics;
	this_Pwindow = Pw;
        this_C_interface = Pw_C_interface;
        boxFont = new Font("Helvetica", Font.PLAIN, 12);
        portFont = new Font("Helvetica", Font.PLAIN, 10);
        fmetrics = getFontMetrics(boxFont);
        boxL = fmetrics.charWidth('m');
        boxH = fmetrics.getHeight();
        fmetrics = getFontMetrics(portFont);
        portL = fmetrics.charWidth('m');
        portH = fmetrics.getHeight();
    }

    private void setColor(Graphics g, int col) {
        switch (col) {
        case White:
            g.setColor(Color.white);
            break;
        case Blue:
            g.setColor(Color.blue);
            break;
        case Red:
            g.setColor(Color.red);
            break;
        case Green:
            g.setColor(Color.green);
            break;
        case Purple:
            g.setColor(new Color(255, 0, 255));
            break;
        case Gray:
            g.setColor(new Color(255, 0, 255));
            break;
        case Black:
            g.setColor(Color.black);
        }
    }

    // Accessor for the typeFont
    public void selectTypeFont(int typeF) {
        typeFont = typeF;
    }

    public int sizeFont() {
        if (typeFont == UserActions.J_box_police)
            return boxFont.getSize();
        if (typeFont == UserActions.J_port_police)
            return portFont.getSize();
        return 12;
    }

    // Gets the name of the font in String
    public String nameFont() {
        if (typeFont == UserActions.J_box_police)
            return new String(boxFont.getName());
        if (typeFont == UserActions.J_port_police)
            return new String(portFont.getName());
        return new String("Helvetica");
    }

    /** Modify boxFont or portFont and the width and height of the font.
	This method is called by ChooseFont when the user has clicked on the button OK
    */
    public void fontModify(Font fontSelected, int l, int h) {
        if (typeFont == UserActions.J_box_police) {
            boxFont = new Font(fontSelected.getAttributes());
            boxH = h;
            boxL = l;
        } else if (typeFont == UserActions.J_port_police) {
            portFont = new Font(fontSelected.getAttributes());
            portH = h;
            portL = l;
        }
        repaint();
    }

    public int getSizeFontH(int typeF) {
        int val = 0;
        ;
        if (typeF == UserActions.J_box_police)
            val = boxH;
        else if (typeF == UserActions.J_port_police)
            val = portH;
        return val;
    }

    public int getSizeFontL(int typeF) {
        int val = 0;
        ;
        if (typeF == UserActions.J_box_police)
            val = boxL;
        else if (typeF == UserActions.J_port_police)
            val = portL;
        return val;
    }

    public void paintComponent(Graphics g) {
        super.paintComponent(g);
	// We need the redraw with the g Graphics in parameter. As soon as we get out from the paintComponent method, g is unallocated.
        gc = g; 
        if (gridState) drawGrid();
        this_C_interface.displayTree();
    }

    public void setGridState(boolean b) {
        gridState = b;
    }

    public void setGridStroke(int n) {
        gridStroke = n;
    }

    public void setGridWidth(int n) {
        largGrid = n;
    }

    public void setScale(double sc) {
        scale = sc;
        repaint();
    }

    public void setMoveMouseEvent() {
        isMouseMovetraited = true;
    }

    public void resetMoveMouseEvent() {
        isMouseMovetraited = false;
    }

    public int getXOrd() {
        return (x);
    }

    public int getYOrd() {
        return (y);
    }

    public int getMouseButton() {
        return (MouseButtonVal);
    }

    public boolean isMouseButtonClicked() {
        w = 0;
        h = 0;
        w_sav = 0;
        h_sav = 0;
        boolean b = MouseButtonState;
        MouseButtonState = false;
        return (b);
    }

    public void drawSelectRect_() {
        sx_sav = Math.min(x, w_sav);
        int sx = Math.min(x, w);
        sy_sav = Math.min(y, h_sav);
        int sy = Math.min(y, h);
        sw_sav = Math.abs(x - w_sav);
        sh_sav = Math.abs(y - h_sav);
        int sw = Math.abs(x - w);
        int sh = Math.abs(y - h);
        if (w != -1) {
            if (compteur != 0) {
                gc.setColor(Color.white);
                gc.drawRect(sx_sav, sy_sav, sw_sav, sh_sav);
            }
            compteur = compteur + 1;
            gc.setColor(Color.black);
            gc.drawRect(sx, sy, sw, sh);
        }
    }

    public void clearRectangle(int x, int y, int w, int h) {
        gc.setColor(Color.white);
        gc.clearRect((int) (x * scale), (int) (y * scale), (int) (w * scale),
                     (int) (h * scale));
    }

    /**
      * Draws a rectangle
      * @param x: x coordinate of the center of the rectangle
      * @param y: y coordinate of the center of the rectangle
      * @param w: width
      * @param h: height
      * @param col: color of the line
      * @param t: thickness of the line
      */
    public void drawRectangle(int x, int y, int w, int h, int col, int t) {
        Graphics2D g = (Graphics2D) gc;
        if (StrokeWidth != 1) 
	    g.setStroke(new BasicStroke(StrokeWidth));
        setColor(gc, col);
        gc.drawRect((int) (x * scale), (int) (y * scale), (int) (w * scale), (int) (h * scale));
        if (StrokeWidth != 1) 
	    g.setStroke(new BasicStroke(1));
    }

    public void fillRectangle(int x, int y, int w, int h, int col) {
        if ((col == Blue) || (col == Red)) { // for selected boxes
            gc.setXORMode(Color.yellow);
            gc.fillRect((int) (x * scale), (int) (y * scale),
                        (int) (w * scale), (int) (h * scale));
        } else if (col == Gray) { // for halos
            // saves the color composition
            Composite originalComposite = gc2.getComposite();
            gc2.setPaint(Color.blue);
            /*  To draw the color with transparency : 0.3F is the transparency
		degree : 0.0F = completely transparent, 1.0F = completely opaque */
            gc2.setComposite(AlphaComposite.getInstance(
                                 AlphaComposite.SRC_OVER, 0.3F));
            gc2.fillRect((int) (x * scale), (int) (y * scale),
                         (int) (w * scale), (int) (h * scale));
            gc2.setComposite(originalComposite);
        } else {
            setColor(gc, col);
            gc.fillRect((int) (x * scale), (int) (y * scale),
                        (int) (w * scale), (int) (h * scale));
        }
        gc.setPaintMode();
    }

    public void fillPolygon(int[] xPoints, int[] yPoints, int nbPoints,
                            int col, int epaisseur) {
        int i;
        setColor(gc, col);
        for (i = 0; i < nbPoints; i++) {
            xPoints[i] = (int) (xPoints[i] * scale);
            yPoints[i] = (int) (yPoints[i] * scale);
        }
        gc.fillPolygon(xPoints, yPoints, nbPoints);
    }

    public void drawPolygon(int[] xPoints, int[] yPoints, int nbPoints,
                            int col, int epaisseur) {
        int i;
        setColor(gc, col);
        for (i = 0; i < nbPoints; i++) {
            xPoints[i] = (int) (xPoints[i] * scale);
            yPoints[i] = (int) (yPoints[i] * scale);
        }
        gc.drawPolygon(xPoints, yPoints, nbPoints);
    }

    public void drawLine(int x1, int y1, int x2, int y2, int col, int thickness) {
        int ep;
        Graphics2D g = (Graphics2D) gc;
        ep = StrokeWidth * thickness; // thickness != 1 for bundle
        if (ep != 1)
            g.setStroke(new BasicStroke(ep));
        setColor(g, col);
        g.drawLine((int) (x1 * scale), (int) (y1 * scale), (int) (x2 * scale), (int) (y2 * scale));
        if (ep != 1)
            g.setStroke(new BasicStroke(1));
    }

    public void drawPolyline(int[] xPoints, int[] yPoints, int nbPoints,
                             int col, int epaisseur) {
        Graphics2D g;
        int i, ep;

        ep = StrokeWidth * epaisseur; // epaisseur != 1 for bundle
        g = (Graphics2D) gc;
        if (ep != 1)
            g.setStroke(new BasicStroke(ep));
        setColor(g, col);
        for (i = 0; i < nbPoints; i++) {
            xPoints[i] = (int) (xPoints[i] * scale);
            yPoints[i] = (int) (yPoints[i] * scale);
        }
        g.drawPolyline(xPoints, yPoints, nbPoints);
        if (ep != 1)
            g.setStroke(new BasicStroke(1)); // Why ?(LB)
    }

    public void drawString(String chaine, int x, int y, int col, int typePolice) {
        setColor(gc, col);
        if (typePolice == UserActions.J_box_police)
            gc.setFont(boxFont);
        else
            gc.setFont(portFont);
        gc.drawString(chaine, (int) (x * scale), (int) (y * scale));
    }

    public void exit() {
        gc.dispose();
    }

    public void init() {
        gc = getGraphics();
        gc2 = (Graphics2D) getGraphics();
        gc2.setStroke(new BasicStroke(2)); // sets the lines width
        CanvasListener canvaslistener = new CanvasListener(this);
        addMouseListener(canvaslistener);
        addMouseMotionListener(canvaslistener);
    }

    public void drawGrid() {
        Graphics2D g = (Graphics2D) gc;
        g.setStroke(new BasicStroke(gridStroke));
        g.setColor(getMyColor1());

        for (int i = largGrid; i < 1000; i = largGrid + i)
            g.drawLine((int) (i * scale), (int) (0 * scale), (int) (i * scale),
                       (int) (800 * scale));

        for (int j = largGrid; j < 800; j = largGrid + j)
            g.drawLine((int) (0 * scale), (int) (j * scale),
                       (int) (1000 * scale), (int) (j * scale));
        if (largGrid != 1)
            g.setStroke(new BasicStroke(1)); // sets the lines width to 1
    }

    public void drawImage(String name, int x, int y) {
        int i = 0;
        boolean trouve = false;
        boolean erreur = false;

        // searching the image in the list "libButton".
        while (!trouve) {
            if (((LibButton) listButLib.get(i)).getName().equals(name))
                trouve = true;
            else if (listButLib.size() == i) {
                trouve = true;
                erreur = true;
                System.out.println("Error, Process and its icon not found!\n");
            } else
                i++;
        }
        if (trouve && !erreur) {
            ImageIcon icon;
            icon = ((LibButton) listButLib.get(i)).getIconLibButton();
            int width = icon.getIconWidth();
            int height = icon.getIconHeight();
            Image imProc = icon.getImage();
            gc.drawImage(imProc, (int) (x * scale), (int) (y * scale), (int) (width * scale), (int) (height * scale), this);
        }
    }

    public void mousePressed(MouseEvent e) {
        gc = getGraphics();
        gc2 = (Graphics2D) getGraphics();
        gc2.setStroke(new BasicStroke(2)); // sets the lines width

        if (SwingUtilities.isRightMouseButton(e))
            MouseButtonVal = C_right; // right button
        else if (SwingUtilities.isLeftMouseButton(e))
            MouseButtonVal = C_left;
        else if (SwingUtilities.isMiddleMouseButton(e))
            if ((e.getModifiers() & MouseEvent.CTRL_MASK) != 0)
                MouseButtonVal = C_ctrl_middle;// middle button + CTRL
            else
                MouseButtonVal = C_middle;//  middle button
        x = (int) (e.getX() / scale);
        y = (int) (e.getY() / scale);
        MouseButtonState = true;
	// System.out.println("DEBUG ==>> sig_iterate in GraphCanvas::MousePressed"); 
	this_C_interface.sig_iterate();
	// System.out.println("DEBUG ==>> sig_iterate in GraphCanvas::MousePressed DONE"); 
    }

    public void mouseMoved(MouseEvent e) {
        if (isMouseMovetraited) {
            int x_in = (int) (e.getX() / scale);
            int y_in = (int) (e.getY() / scale);
            if (compteur != 0) {
                w_sav = w;
                h_sav = h;
            }
            w = x_in;
            h = y_in;
            this_C_interface.getMouseMoveCoord(x_in, y_in);
        }
    }

    public void actionPerformedGrid(int userAction) {
        if (userAction == UserActions.J_grid_none)
            gridState = false;
        else {
            gridState = true;
            gridStroke = 1;
            if (userAction == UserActions.J_grid_1)
                largGrid = 20;
            else if (userAction == UserActions.J_grid_2)
                largGrid = 40;
            else
                largGrid = 80;
        }
        repaint();
    }

    public void actionPerformedStrokeWidth(int userAction) {
        if (userAction == UserActions.J_StrokeWidth_1)
            StrokeWidth = 1;
        else if (userAction == UserActions.J_StrokeWidth_2)
            StrokeWidth = 2;
        else if (userAction == UserActions.J_StrokeWidth_3)
            StrokeWidth = 3;
        repaint();
    }

}

class CanvasListener extends MouseInputAdapter
{
    private GraphCanvas canvas;

    public CanvasListener(GraphCanvas g) {
        canvas = g;
    }

    public void mousePressed(MouseEvent e) {
        canvas.mousePressed(e);
    }

    public void mouseMoved(MouseEvent e) {
        canvas.mouseMoved(e);
    }

    public void mouseReleased(MouseEvent e) {
    }

    public void mouseEntered(MouseEvent e) {
    }

    public void mouseExited(MouseEvent e) {
    }

    public void mouseClicked(MouseEvent e) {
    }
}
