 /*
   Authors: Loïc Besnard, Patricia Bournai, Thierry Gautier, Paul Le Guernic.
   Copyright (C) by INRIA-CNRS - developed by Espresso project - GPL v2 License - 2010.

   Contact: polychronycontact@listes.irisa.fr

   This file is part of Polychrony Signal GUI (http://www.irisa.fr/espresso/polychrony).

   Polychrony Signal GUI is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License Version 2 as published by the Free
   Software Foundation.

   Polychrony Signal GUI is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License Version 2
   for more details.

   You should have received a copy of the GNU General Public License Version 2
   along with Polychrony Signal GUI; see the file COPYING. If not, consult the url 
    http://www.gnu.org/licenses/gpl-2.0.html, or write to the Free
   Software Foundation, 51 Franklin Street, Fifth Floor, Boston, MA
   02110-1301, USA.
 */


package common;

import java.io.File;
import common.PolychronyWindow;


/**
 * JAVA <--> C++, C interface. The GUI is written in JAVA and Signal environment
 * (compiler) in C++, C languages. All the compiler modules are defined in the
 * library called libSIGNAL.so (system dependant) Note that JAVA calls C world
 * and vice-versa.
 */
public class LiaisonSignal {
    
    native long n_globalInit();
    
    native long n_localInit();
    
    native long n_SignalProcCreate(long pSupervis, long father, String name, int category, int index);
    
    native long n_SignalModuleCreate(long pSupervis, long father, String name);
    
    native long n_SignalRootCreate(long pSupervis, String name);

    native void n_SignalTreeDelete(long pSupervis, long p, boolean detach);
    
    native long n_SignalTreeCopy(long pSupervis, long p);
    
    native void n_SignalTreeInsert(long pSupervis, long procFather,
                                   long procChild, int index);
    
    native long n_getSignalChildRef(long pSupervis, long procFather, long index);
    
    native long n_getSignalTreeArity(long proc);
    
    native void n_SignalTreeSelect(long pSupervis, long p);
    
    native void n_SignalChangeProcName(long pSupervis, long p, String name);
    
//    native int n_save_ntrees(long pSupervis, long ntreesSource, String name);
    
    native boolean n_sig_iterate(long pSupervis);
    
    native void n_displayTree(long pSupervis);
    
    native long n_getRacine(long pSupervis);
    
    native void n_getMouseMoveCoord(long pSupervis, long x, long y);
    
    native void n_set_COMPILER_OPTION(long pSupervis, long i);
    
    native void n_endAutoComp(long pSupervis, boolean ok, String voptions);
    
    native void n_okParam(long pSupervis, String s);
    
    native void n_WithoutParam(long pSupervis);
    
    native String n_getNameProc(long pSupervis, long signalRef);
    
    native void n_SignalRaz(long pSupervis);
    
    native void n_SignalExit(long pSupervis);
    
    native int n_loadFile(long pSupervis, int action, String fn);
    
    native boolean n_isProcess(long pSupervis, long signalRef);
    
    native boolean n_isModel(long pSupervis, long signalRef);
    
    native boolean n_isFunc(long pSupervis, long signalRef);
    
    native boolean n_isNode(long pSupervis, long signalRef);
    
    native boolean n_isAction(long pSupervis, long signalRef);
    
    native boolean n_isModul(long pSupervis, long signalRef);
    
    native String n_getNameGif(long i, long signalRef);
    
    native synchronized String n_getShellVar(String var);
    
    // static { System.loadLibrary("Signal"); }
    static {
        String str = System.getenv ("SignalGUI_obj") + File.separatorChar + System.mapLibraryName("Signal");
	System.load(str);
    }

    static String docDirectory; // directory of the doc files (for the help)

    private PolychronyWindow this_parentPwindow; // cannot be null
    private CompilerWin compiler;
    private LibBar libBar; // bar from the currently used library
    private LibButton libButt; // button of the currently used libBar
    private PortNameWin portNameWin;
    private long pSupervis; // C++ object corresponding to one's window supervisor

    // Methods called from JAVA to C
    public synchronized void globalInit() {
        pSupervis = n_globalInit();
    }

    public synchronized void localInit() {
        pSupervis = n_localInit();
    }

    public synchronized String getShellVar(String var) {
        return n_getShellVar(var);
    }

    public synchronized long SignalProcCreate(long father, String name, int category, int index) {
        return n_SignalProcCreate(pSupervis, father, name, category, index);
    }

    public synchronized long SignalModuleCreate(long father, String name) {
        return n_SignalModuleCreate(pSupervis, father, name);
    }

    public synchronized long SignalRootCreate(String name) {
        return n_SignalRootCreate(pSupervis, name);
    }

    public synchronized void SignalTreeDelete(long p, boolean detach) {
        n_SignalTreeDelete(pSupervis, p, detach);
    }

    public synchronized long SignalTreeCopy(long p) {
        return n_SignalTreeCopy(pSupervis, p);
    }

    public synchronized void SignalTreeInsert(long procFather, long procChild, int index) {
        n_SignalTreeInsert(pSupervis, procFather, procChild, index);
    }

    public synchronized long getSignalChildRef(long procFather, long index) {
        return (n_getSignalChildRef(pSupervis, procFather, index));
    }

    public synchronized long getSignalTreeArity(long proc) {
        return (n_getSignalTreeArity(proc));
    }

    public synchronized void SignalTreeSelect(long p) {
        n_SignalTreeSelect(pSupervis, p);
    }

    public synchronized void SignalChangeProcName(long p, String name) {
        n_SignalChangeProcName(pSupervis, p, name);
    }

//    public synchronized int save_ntrees(long ntreesSource, String name); {
//        return (n_save_ntrees(pSupervis, ntreesSource, name));
//    }

    public synchronized boolean sig_iterate() {
	// System.out.println("DEBUG ===== VIA ==== LiaisonSignal::sig_iterate" );
        return (n_sig_iterate(pSupervis));
    }

    public synchronized void displayTree() {
        n_displayTree(pSupervis);
    }

    public synchronized long getRacine() {
        return (n_getRacine(pSupervis));
    }

    public synchronized void getMouseMoveCoord(int x, int y) {
        n_getMouseMoveCoord(pSupervis, x, y);
    }

    public synchronized void set_COMPILER_OPTION(int i) {
        n_set_COMPILER_OPTION(pSupervis, i);
    }

    //public synchronized void endAutoComp(boolean ok) { n_endAutoComp(pSupervis, ok); }

    public synchronized void endAutoComp(boolean ok, String voptions) {
        n_endAutoComp(pSupervis, ok, voptions);
    }

    public synchronized void okParam(String s) {
        n_okParam(pSupervis, s);
    }

    public synchronized void WithoutParam() {
        n_WithoutParam(pSupervis);
    }

    public synchronized String getNameProc(long signalRef) {
        return (n_getNameProc(pSupervis, signalRef));
    }

    public synchronized void SignalExit() {
        n_SignalExit(pSupervis);
    }

    public synchronized void SignalRaz() {
        n_SignalRaz(pSupervis);
    }

//    public synchronized int loadFile(int action, String fn) {
//	System.out.println("DEBUG ===== VIA ==== LiaisonSignal.java loadFile");
//        return n_loadFile(pSupervis, action, fn);
//    }

    public synchronized boolean isProcess(long signalRef) {
        return n_isProcess(pSupervis, signalRef);
    }

    public synchronized boolean isFunc(long signalRef) {
        return n_isFunc(pSupervis, signalRef);
    }

    public synchronized boolean isModel(long signalRef) {
        return n_isModel(pSupervis, signalRef);
    }

    public synchronized boolean isNode(long signalRef) {
        return n_isNode(pSupervis, signalRef);
    }

    public synchronized boolean isAction(long signalRef) {
        return n_isAction(pSupervis, signalRef);
    }

    public synchronized boolean isModul(long signalRef) {
        return n_isModul(pSupervis, signalRef);
    }

    public synchronized String getNameGif(long signalref, long i) {
        return n_getNameGif(signalref, i);
    }

    public void Sav(PolychronyWindow g, CompilerWin f, String docDir) {
        this_parentPwindow = g;
        compiler = f;
        docDirectory = docDir;
    }

    // ======================================================================
    // Methods called from C to JAVA
    // ======================================================================

    public void refresh(boolean hasChanged) {
	if ( this_parentPwindow != null ) {
	    if ( hasChanged )
		this_parentPwindow.refresh(true);
	    else
		this_parentPwindow.refresh(); }
    }
    
    public void drawPolyline(int[] x_points, int[] y_points, int nbPoints,
                             int coul, int ep) {
        this_parentPwindow.drawPolyline(x_points, y_points, nbPoints, coul, ep);
    }

    public void drawLine(int x1, int y1, int x2, int y2, int coul, int ep) {
        this_parentPwindow.drawLine(x1, y1, x2, y2, coul, ep);
    }

    public void fillPolygon(int[] xPoints, int[] yPoints, int nPoints,
                            int coul, int ep) {
        this_parentPwindow.fillPolygon(xPoints, yPoints, nPoints, coul, ep);
    }

    public void drawPolygon(int[] xPoints, int[] yPoints, int nPoints,
                            int coul, int ep) {
        this_parentPwindow.drawPolygon(xPoints, yPoints, nPoints, coul, ep);
    }

    public void drawRectangle(int x, int y, int w, int h, int coul, int ep) {
        this_parentPwindow.drawRectangle(x, y, w, h, coul, ep);
    }

    public void fillRectangle(int x, int y, int w, int h, int coul) {
        this_parentPwindow.fillRectangle(x, y, w, h, coul);
    }

    public void drawString(String chaine, int x, int y, int coul, int typePolice) {
        this_parentPwindow.drawString(chaine, x, y, coul, typePolice);
    }

    public void drawImage(String name, int x, int y) {
        this_parentPwindow.drawImage(name, x, y);
    }

    public int readMenuButton() {
        return this_parentPwindow.read_clickedAction();
    }

    public void razMenuButton() {
        this_parentPwindow.raz_selectedAction();
    }

    public boolean getNamesState() {
        return (this_parentPwindow.get_portNameVisibility());
    }

    public boolean getDetailState() {
        return (this_parentPwindow.getdetailsState());
    }

    public int getMouseButton() {
        return this_parentPwindow.get_GEwindowMouseButton(); 
    }

    public boolean isMouseButtonClicked() {
        return (this_parentPwindow.is_GEwindowMouseButtonClicked());
    }

    public int getXOrd() {
        return (this_parentPwindow.getXOrd());
    }

    public int getYOrd() {
        return (this_parentPwindow.getYOrd());
    }

    public void setMoveMouseEvent() {
        this_parentPwindow.set_GEwindowMoveMouseEvent();
    }

    public void resetMoveMouseEvent() {
        this_parentPwindow.reset_GEwindowMoveMouseEvent();
    }

    public void textHide(int w) {
	this_parentPwindow.hide_Twindow(w);
    }

    public void textShow(int w) {
	this_parentPwindow.show_Twindow(w);
    }

    public void appendText(int w, String s) {
	this_parentPwindow.append_text2Twindow(w, s);
    }

    public String getText(int w) {
	return this_parentPwindow.get_TwindowContent(w);
    }

    public int getColumns(int w) {
	return this_parentPwindow.get_TwindowCharWidth(w);
    }

    public void resetText(int w) {
	this_parentPwindow.clear_Twindow(w);
    }

    public int getLastActiveTextWin() {
	return this_parentPwindow.get_activeTwindowType();
    }

    public void showNamePort(String name) {
        portNameWin = new PortNameWin("     " + name);
        portNameWin.setVisible(true);
    }

    public void showPortHide() {
        portNameWin.dispose();
    }

    public byte[] getText1(String x) {
	// System.out.println("DEBUG entering/exiting LiaisonSignal::getText1");
        return x.getBytes();
    }

    public void print() {
        this_parentPwindow.print_GEwindow();
    }

    /**
       @return 0 when operation is canceled or errored
       @return -1 when a file name is provided and the current action is load and current complement is J_before
       @return 1 when a file name is provided
    */
    public int isBrowserOk(boolean isGraphic, boolean openDialog) {
        return (this_parentPwindow.browser_directory_file_path(isGraphic, openDialog));
    }

    public String getBrowserFileName() {
        return (this_parentPwindow.getBrowserFileName());
    }

    public String getCurrentPath() {
        return (this_parentPwindow.getCurrentPath());
    }

    public int getUserChoice() {
        return (this_parentPwindow.getUserChoice());
    }

    public String getString1() {
        return (this_parentPwindow.getString1());
    }

    public String getString2() {
        return (this_parentPwindow.getString2());
    }

    public void displayError(int noErr) {
        this_parentPwindow.displayError(noErr);
    }

    public void displayParsingError(String errorString) {
        this_parentPwindow.displayParsingError(errorString);
    }

    public void quitApplication() {
        this_parentPwindow.exit_Polychrony();
    }

    public void displayConfirm(int action) {
        this_parentPwindow.displayConfirm(action);
    }

    public void displayCurrentAction(int action) {
        this_parentPwindow.displayCurrentAction(action);
    }

    public void displayFont() {
        this_parentPwindow.displayFont();
    }

    public int getSizeFontH(int type) {
        return this_parentPwindow.getSizeFontH(type);
    }

    public int getSizeFontL(int type) {
        return this_parentPwindow.getSizeFontL(type);
    }

    public void displayCompilerFunct() {
        compiler.displayCompilerFunct();
    }

    public void displayAutoCompilerFunct() {
        compiler.displayAutoCompilerFunct();
    }

    public void displayCompilerDiag() {
        compiler.displayCompilerDiag();
    }

    public int getCompilerFunctionality() {
        return (compiler.getCompilerFunctionality());
    }

    public void setColorCompilerFunct(int func, int col) {
        compiler.getCompilerFunct().setColorFunct(func, col);
    }

    public void initSelectedNode(boolean afterCompil) {
        this_parentPwindow.initSelectedNode(afterCompil);
    }

    public void upTempSelectedNode() {
        this_parentPwindow.upTempSelectedNode();
    }

    public void SetCurrentTree(int iBox) { 
        this_parentPwindow.setCurrentTree(iBox);
    }

    public void deleteSelectedNode() {
        this_parentPwindow.deleteSelectedNode();
    }
    public void appendNewJNode(String nom, int SignalTree, boolean setInAtEnd) {
        this_parentPwindow.appendNewJNode(nom, SignalTree, setInAtEnd);
    }

    public void helpShow() {
        new Help(docDirectory);
    }

    public boolean isLoadedLibrary(String fname) {
        return this_parentPwindow.isLoadedLibrary(fname);
    }

    public int getIndexLibrary(String fname) {
        return this_parentPwindow.getIndexLibrary(fname);
    }

    public void createLibBar(long signalRef, String fname, String fnamews, String libName) {
        this_parentPwindow.createLibBar(signalRef, fname, fnamews, libName);
    }

    public int getIconIndex() {
        return libButt.getNum();
    }

    public int getHightIcon() {
        return libButt.getIconLibButton().getIconHeight();
    }

    public int getLenghtIcon() {
        return libButt.getIconLibButton().getIconWidth();
    }

    public void removeSelectedLib(String fname) {
        this_parentPwindow.removeLibrary(fname);
    }

    public void install_library(int vtree, int vindex) {
        this_parentPwindow.install_library(vtree, vindex);
    }

    public String getSelectedFilenameLibrary() {
        return libBar.getFilename();
    }

    public long getRefLib() {
        return libBar.getSignalRefTree();
    }

    public CompilerWin getCompilerWin() {
        return compiler;
    }

    public void sigIterateLibButton(LibBar barre, LibButton but, boolean copyIcon) {
        libBar = barre;
        libButt = but;
	if ( copyIcon )
	    this_parentPwindow.set_clickedAction(UserActions.J_copy_Icon);
	else
	    this_parentPwindow.set_clickedAction(UserActions.J_info_Icon);
        if (sig_iterate());
    }

//    public void actionPerformedMiddleLibButton(LibBar barre, LibButton but) {
//        libBar = barre;
//        libButt = but;
//        this_parentPwindow.set_clickedAction(UserActions.J_info_Icon);
//	if (sig_iterate());
//    }

    public void ListernerPopupLibBar(LibBar barre, int code) {
        libBar = barre;
        this_parentPwindow.set_clickedAction((code == 1) ? UserActions.J_removeLib : UserActions.J_reloadLib);
        if (sig_iterate());
    }
}
