 /*
   Authors: Loïc Besnard, Patricia Bournai, Thierry Gautier, Paul Le Guernic.
   Copyright (C) by INRIA-CNRS - developed by Espresso project - GPL v2 License - 2010.

   Contact: polychronycontact@listes.irisa.fr

   This file is part of Polychrony Signal GUI (http://www.irisa.fr/espresso/polychrony).

   Polychrony Signal GUI is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License Version 2 as published by the Free
   Software Foundation.

   Polychrony Signal GUI is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License Version 2
   for more details.

   You should have received a copy of the GNU General Public License Version 2
   along with Polychrony Signal GUI; see the file COPYING. If not, consult the url 
    http://www.gnu.org/licenses/gpl-2.0.html, or write to the Free
   Software Foundation, 51 Franklin Street, Fifth Floor, Boston, MA
   02110-1301, USA.
 */


package common;

import java.awt.event.MouseEvent;
import java.io.File;

import javax.swing.ImageIcon;
import javax.swing.JButton;
import javax.swing.SwingUtilities;
import javax.swing.event.MouseInputAdapter;


/** Class creating a button containing a number, an icon and a name. The number
    allows us to know which process the button is associated with, in the
    reference tree signalRef from the LibBar class.
 */
public class LibButton extends JButton {
    private static final long serialVersionUID = 1L;
    private int num;
    private String name;
    private LiaisonSignal this_C_interface;
    private LibBar this_libBar;
    private ImageIcon icon;

    /** Individual button in user defined list of buttons
	called when installing a LibBar
     */
    public LibButton(String nam, int n, LiaisonSignal ls, LibBar bar,
                     String fname, String defaultIconDir) {
        String imagename;
        num = n;
        name = nam;
        this_C_interface = ls;
        this_libBar = bar;
        imagename = fname + "_ICO" + File.separator + name + ".gif";
        File inputFile = new File(imagename);
        if (!inputFile.exists())
            imagename = defaultIconDir + "undefIcon.gif";

        icon = new ImageIcon(imagename);
        setIcon(icon);
        setToolTipText(name);

        ActLibButt ActLibButtListener = new ActLibButt(this_libBar, this, this_C_interface);
        addMouseListener(ActLibButtListener);
        addMouseMotionListener(ActLibButtListener);

    }

    public int getNum() {
        return num;
    }

    public ImageIcon getIconLibButton() {
        return icon;
    }

    public String getName() {
        return name;
    }

}

class ActLibButt extends MouseInputAdapter {
    LiaisonSignal this_C_interface;
    LibButton this_libButton;
    LibBar this_libBar;

    public ActLibButt(LibBar bar, LibButton b, LiaisonSignal ls) {
        this_C_interface = ls;
        this_libButton = b;
        this_libBar = bar;
    }

    public void mousePressed(MouseEvent e) {
        /*
         * if (SwingUtilities.isRightMouseButton(e)): no action 
         * else call to this_C_interface.sigIterateLibButton(this_libBar, this_libButton, userAction copy or info);
         */
        if (SwingUtilities.isLeftMouseButton(e))
            // this_C_interface.sigIterateLibButton(this_libBar, this_libButton)
	    this_C_interface.sigIterateLibButton(this_libBar, this_libButton, true); // UserActions.J_copy_Icon
        else if (SwingUtilities.isMiddleMouseButton(e))
	    // this_C_interface.actionPerformedMiddleLibButton(this_libBar, this_libButton)
            this_C_interface.sigIterateLibButton(this_libBar, this_libButton, false); // UserActions.J_info_Icon
    }
}
