 /*
   Authors: Loïc Besnard, Patricia Bournai, Thierry Gautier, Paul Le Guernic.
   Copyright (C) by INRIA-CNRS - developed by Espresso project - GPL v2 License - 2010.

   Contact: polychronycontact@listes.irisa.fr

   This file is part of Polychrony Signal GUI (http://www.irisa.fr/espresso/polychrony).

   Polychrony Signal GUI is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License Version 2 as published by the Free
   Software Foundation.

   Polychrony Signal GUI is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License Version 2
   for more details.

   You should have received a copy of the GNU General Public License Version 2
   along with Polychrony Signal GUI; see the file COPYING. If not, consult the url 
    http://www.gnu.org/licenses/gpl-2.0.html, or write to the Free
   Software Foundation, 51 Franklin Street, Fifth Floor, Boston, MA
   02110-1301, USA.
 */


package common;

import java.awt.Container;
import java.awt.TextArea;
import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Font;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.awt.event.WindowListener;

import javax.swing.JButton;
import javax.swing.JFrame;
import javax.swing.JPanel;


public class ParamWindow extends JFrame {
    private static final long serialVersionUID = 1L;
    private LiaisonSignal LS;
    private TextArea text = new TextArea();
    private String textCopy = "";

    private Color getMyColor1() {
        return pKGUIColors.Color1;
    }

    public ParamWindow(LiaisonSignal l, int x0, int y0, int width, int height,
                       String title) {
        LS = l;
        Container frame = getContentPane();
        frame.setLayout(new BorderLayout());
        JPanel panel = new JPanel();
        JButton okButton = new JButton(" OK ");
        ParamListener myListener = new ParamListener(this);
        okButton.addActionListener(myListener);
        panel.add(okButton, BorderLayout.CENTER);

        JButton NPButton = new JButton(" No parameters ");
        ParamListenerNoParameters npListener = new ParamListenerNoParameters(
            this);
        NPButton.addActionListener(npListener);
        panel.add(NPButton, BorderLayout.WEST);

        setBackground(getMyColor1());
        // Added at the bottom of the buttons bar
        frame.add(panel, BorderLayout.SOUTH);
        // Added in the center of the text input zone
        text.setBackground(Color.white);
        frame.add(text, BorderLayout.CENTER);
        text.setFont(new Font("Helvetica", Font.PLAIN, 14));
        setLocation(x0, y0);
        setSize(width, height);
        setTitle(title);
        pack();
        setVisible(false);
        // to close the window
        WindowListener closeWin = new WindowAdapter() {
            public void windowClosing(WindowEvent e) {
                setVisible(false);
                LS.WithoutParam();
            }
        };
        addWindowListener(closeWin);

    }

    public void append(String s) {
        text.append(s);
    }

    public void setCaretPosition(int n) {
        text.setCaretPosition(n);
    }

    public void resetText() {
        text.setText("");
    }

    public String getText() {
        return (text.getText());
    }

    public int getColumns() {
        return (text.getColumns());
    }

    public LiaisonSignal getPkLink() {
        return LS;
    }
}

class ParamListener implements ActionListener {
    ParamWindow parent;

    public ParamListener(ParamWindow w) {
        parent = w;
    }

    public void actionPerformed(ActionEvent e) {
	// JButton source = (JButton) e.getSource();
        // String label = source.getText();
        parent.setVisible(false);
        parent.getPkLink().okParam(parent.getText());
    }
}

class ParamListenerNoParameters implements ActionListener {
    ParamWindow parent;

    public ParamListenerNoParameters(ParamWindow w) {
        parent = w;
    }

    public void actionPerformed(ActionEvent e) {
        // parent.hide(); // closing the window.
        parent.setVisible(false);
        parent.getPkLink().WithoutParam();
    }
}
