/*
   Authors: Loïc Besnard, Patricia Bournai, Thierry Gautier, Paul Le Guernic.
   Copyright (C) by INRIA-CNRS - developed by Espresso project - GPL v2 License - 2010.

   Contact: polychronycontact@listes.irisa.fr

   This file is part of Polychrony Signal GUI (http://www.irisa.fr/espresso/polychrony).

   Polychrony Signal GUI is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License Version 2 as published by the Free
   Software Foundation.

   Polychrony Signal GUI is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License Version 2
   for more details.

   You should have received a copy of the GNU General Public License Version 2
   along with Polychrony Signal GUI; see the file COPYING. If not, consult the url 
    http://www.gnu.org/licenses/gpl-2.0.html, or write to the Free
   Software Foundation, 51 Franklin Street, Fifth Floor, Boston, MA
   02110-1301, USA.
 */
package gui_dependent;

import java.awt.Frame;
import java.io.File;
import java.io.IOException;
import java.lang.reflect.Field;
import java.net.URL;

import org.eclipse.core.runtime.FileLocator;
import org.eclipse.core.runtime.Platform;
import common.PolychronyWindow;

/**
 * Polychrony GUI main graph editing tool window.
 */
public class Polychrony {
    private static PolychronyWindow the_mainPolychronyWindow= null;
    
    public PolychronyWindow getGw() {
	return the_mainPolychronyWindow;
    }
    
    public Polychrony(Frame frame) {
	// this.frame = frame;
	String SignalGUI = System.getenv ("SignalGUI_bin"); // @ of the Signal GUI binary.
	String[] args = {".", SignalGUI + File.separatorChar +"images", SignalGUI + File.separatorChar + "doc" };
	addToJavaLibraryPath(SignalGUI);
	addToJavaLibraryPath(System.getenv ("SignalGUI_obj"));
	the_mainPolychronyWindow = new PolychronyWindow(args[0], args[1], args[2], frame);
    }
    
    public void reset_polychrony() 
    {
	the_mainPolychronyWindow.reset_polychrony();
    }
    
    public void setFrame(Frame frame) { 
	the_mainPolychronyWindow.setFrame(frame);
    }
    
    /**
     * With this method, we obtain the path of the workspace of a given plugin.
     * 
     * @param pluginName
     *            the name of the plugin whose path we want
     * @return the path of the workspace of this plugin
     */
    public String getPluginPath(String pluginName) {
	String returnValue = "";
	URL pluginRootUrl = Platform.getBundle(pluginName).getEntry("");
	try {
	    returnValue = new File(FileLocator.toFileURL(pluginRootUrl).getFile()).getAbsolutePath();
	} catch (IOException e) {
	    e.printStackTrace();
	}
	return returnValue;
    }
    
    /**
     * Adds a new directory in java.library.path.
     * 
     * @param directory
     *            Le nouveau répertoire à ajouter.
     */
    private void addToJavaLibraryPath(String directory) {
	final String LIBRARY_PATH = "java.library.path";
	String javaLibraryPath = System.getProperty(LIBRARY_PATH);
	System.setProperty(LIBRARY_PATH, javaLibraryPath + File.pathSeparatorChar + directory);
	resetJavaLibraryPath();
    }
    
    /**
     * Deletes the "java.library.path" cache. This will force the evaluation of
     * its value the next time a library is loaded.
     * 
     * This action is specific to the Sun JVM and may not work on other virtual
     * machines.
     */
    private void resetJavaLibraryPath() {
	synchronized (Runtime.getRuntime()) {
	    try {
		Field field = ClassLoader.class.getDeclaredField("usr_paths");
		field.setAccessible(true);
		field.set(null, null);
		field = ClassLoader.class.getDeclaredField("sys_paths");
		field.setAccessible(true);
		field.set(null, null);
	    } catch (NoSuchFieldException e) {
		e.printStackTrace();
	    } catch (IllegalAccessException e) {
		e.printStackTrace();
	    }
	}
    }
    
}
