package org.eclipse.pop.gui.popup.actions;

import java.util.Iterator;

import org.eclipse.core.resources.IFile;
import org.eclipse.jface.action.IAction;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.StructuredSelection;
import org.eclipse.pop.gui.views.PolychronyView;
import org.eclipse.ui.IObjectActionDelegate;
import org.eclipse.ui.IViewPart;
import org.eclipse.ui.IViewReference;
import org.eclipse.ui.IWorkbenchPage;
import org.eclipse.ui.IWorkbenchPart;
import org.eclipse.ui.PartInitException;
import org.eclipse.ui.PlatformUI;

import common.PolychronyWindow;

public class OpenAction implements IObjectActionDelegate {
	
	private StructuredSelection	currentSelection;
	private IFile				gpkFile;
	private boolean				BDEBUG	= false;
	private final String ID = "org.eclipse.pop.gui.views.PolychronyView";
	/**
	 * Constructor for Action1.
	 */
	public OpenAction() {
		super();
	}
	
	/**
	 * @see IObjectActionDelegate#setActivePart(IAction, IWorkbenchPart)
	 */
	public void setActivePart(IAction action, IWorkbenchPart targetPart) {}
	
	/**
	 * @see IActionDelegate#run(IAction)
	 */
	public void run(IAction action) {
		final String fileName = gpkFile.getLocation().toOSString();
		
		IViewPart viewPart;

		IWorkbenchPage page = PlatformUI.getWorkbench().getActiveWorkbenchWindow().getActivePage();

		if (page == null) viewPart = null; 	else { viewPart = page.findView(ID); printDebug(page);}

		if (viewPart != null) {
			if (BDEBUG)  System.err.println("Open Action cas 1");
			javax.swing.SwingUtilities.invokeLater(new Runnable() {
				public void run() {
					load(fileName);
				}
			});
		}
		else {
			if (BDEBUG) System.err.println("The GUI view is not opened, I open it for you.!!!!");
			try {
				page.showView(ID);				 
				load(fileName);
			}
			catch (PartInitException e1) {
				e1.printStackTrace();
			} 
		}
	}
	private void load(String fileName) {
		 PolychronyView.getPolychrony().getGw().initialLoadRoot(fileName, true, true);
	}

	/** 
	 * DEBUG it prints the current opened views of the operand page.
	 */
	private void printDebug(IWorkbenchPage page) { 
		if (BDEBUG)
		{
			IViewReference[] viewReferences = page.getViewReferences();	 
			for (IViewReference ivr : viewReferences) {
				System.err.println(" -- class OpenAction ... " + ivr.getId() );  
			} 
		}
	}

	/**
	 * @see IActionDelegate#selectionChanged(IAction, ISelection)
	 */
	public void selectionChanged(IAction action, ISelection selection) {
		if (selection instanceof StructuredSelection) {
			currentSelection = (StructuredSelection) selection;
			Iterator<?> it = currentSelection.iterator();
			while (it.hasNext()) {
				gpkFile = (IFile) it.next();
			}
		}
	}
}
