package org.eclipse.pop.gui.views;

import java.awt.Frame;

import org.eclipse.jface.viewers.IStructuredContentProvider;
import org.eclipse.jface.viewers.ITableLabelProvider;
import org.eclipse.jface.viewers.LabelProvider;
import org.eclipse.jface.viewers.Viewer;
import org.eclipse.jface.viewers.ViewerSorter;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.awt.SWT_AWT;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.SWT;
import org.eclipse.ui.ISharedImages;
import org.eclipse.ui.IViewPart;
import org.eclipse.ui.IViewReference;
import org.eclipse.ui.IWorkbenchPage;
import org.eclipse.ui.PartInitException;
import org.eclipse.ui.PlatformUI;
import org.eclipse.ui.part.ViewPart;

import gui_dependent.Polychrony;

/**
 * This sample class demonstrates how to plug-in a new workbench view. The view shows data obtained from the model. The
 * sample creates a dummy model on the fly, but a real implementation would connect to the model available either in
 * this or another plug-in (e.g. the workspace). The view is connected to the model using a content provider.
 * <p>
 * The view uses a label provider to define how model objects should be presented in the view. Each view can present the
 * same model objects using different labels and icons, if needed. Alternatively, a single label provider can be shared
 * between views in order to ensure that objects of the same type are presented in the same way everywhere.
 * <p>
 */

public class PolychronyView extends ViewPart {

	private static Polychrony	polychrony=null;
	 
	/**
	 * The constructor.
	 */
	public PolychronyView(){
		 
	};
	
	
	public static Polychrony getPolychrony() {
		return polychrony;
	}
	
	/**
	 * The ID of the view as specified by the extension.
	 */
	
	public static final String	ID	= "org.eclipse.pop.gui.views.PolychronyView";
	/*
	 * The content provider class is responsible for providing objects to the view. It can wrap existing objects in
	 * adapters or simply return objects as-is. These objects may be sensitive to the current input of the view, or
	 * ignore it and always show the same content (like Task List, for example).
	 */

	class ViewContentProvider implements IStructuredContentProvider {
		public void inputChanged(Viewer v, Object oldInput, Object newInput) { }
		public void dispose() {}
		public Object[] getElements(Object parent) { return new String[] { "" };
		}
	}
	
	class ViewLabelProvider extends LabelProvider implements ITableLabelProvider {
		public String getColumnText(Object obj, int index) {return getText(obj);}
		public Image getColumnImage(Object obj, int index) {return getImage(obj);}
		public Image getImage(Object obj) {
			return PlatformUI.getWorkbench().getSharedImages().getImage(ISharedImages.IMG_OBJ_ELEMENT);
		}
	}
	
	class NameSorter extends ViewerSorter {}

	/**
	 * This is a callback that will allow us to create the viewer and initialize it.
	 */
	public void createPartControl(Composite parent) {
		Composite composite = new Composite(parent, SWT.EMBEDDED | SWT.NO_BACKGROUND);	
		Frame frame = SWT_AWT.new_Frame(composite);
		if (polychrony == null) polychrony = new Polychrony(frame); else polychrony.setFrame(frame);
	} 

	 
	/**
	 * Passing the focus request to the viewer's control.
	 */
	public void setFocus() {}
 
	
//	public void dispose() {
//	  //  super.dispose();
//	    IWorkbenchPage page = PlatformUI.getWorkbench().getActiveWorkbenchWindow().getActivePage();
//	    if (page != null) {
//	        IViewReference[] viewReferences = page.getViewReferences();
//	        for (IViewReference ivr : viewReferences) {
//	        	System.err.println(" -- class PolychronyView :: dispose() " + ivr.getId() );
//	            if (ivr.getId().equalsIgnoreCase(ID)
//	                    /*|| ivr.getId().equalsIgnoreCase("more view id if you want to close more than one at a time")*/) {
//	                page.hideView(ivr);
//	            }
//	        }
//	    }
//	}
// 
//	
	
}
